@@ -55,7 +55,7 @@ struct _GstDeviceProviderPrivate
 
   GMutex start_lock;
 
-  gboolean started_count;
+  gint started_count;
 
   GList *hidden_providers;
 };
@@ -84,7 +84,7 @@ static gint private_offset = 0;
 GType
 gst_device_provider_get_type (void)
 {
-  static volatile gsize gst_device_provider_type = 0;
+  static gsize gst_device_provider_type = 0;
 
   if (g_once_init_enter (&gst_device_provider_type)) {
     GType _type;
@@ -165,6 +165,8 @@ gst_device_provider_init (GstDeviceProvider * provider)
 
   g_mutex_init (&provider->priv->start_lock);
 
+  provider->priv->started_count = 0;
+
   provider->priv->bus = gst_bus_new ();
   gst_bus_set_flushing (provider->priv->bus, TRUE);
 }
@@ -417,9 +419,15 @@ gst_device_provider_get_devices (GstDeviceProvider * provider)
     for (item = provider->devices; item; item = item->next)
       devices = g_list_prepend (devices, gst_object_ref (item->data));
     GST_OBJECT_UNLOCK (provider);
-  } else if (klass->probe)
+  } else if (klass->probe) {
+
     devices = klass->probe (provider);
 
+    for (item = devices; item; item = item->next)
+      if (g_object_is_floating (item->data))
+        g_object_ref_sink (item->data);
+  }
+
   g_mutex_unlock (&provider->priv->start_lock);
 
   return devices;
@@ -546,7 +554,7 @@ gst_device_provider_stop (GstDeviceProvider * provider)
  *
  * Retrieves the factory that was used to create this device provider.
  *
- * Returns: (transfer none): the #GstDeviceProviderFactory used for
+ * Returns: (transfer none) (nullable): the #GstDeviceProviderFactory used for
  *     creating this device provider. no refcounting is needed.
  *
  * Since: 1.4
@@ -848,3 +856,24 @@ gst_device_provider_device_changed (GstDeviceProvider * provider,
   gst_bus_post (provider->priv->bus, message);
   gst_object_unparent (GST_OBJECT (changed_device));
 }
+
+/**
+ * gst_device_provider_is_started:
+ * @provider: a #GstDeviceProvider
+ *
+ * This function can be used to know if the @provider was successfully started.
+ *
+ * Since: 1.20
+ */
+gboolean
+gst_device_provider_is_started (GstDeviceProvider * provider)
+{
+  gboolean started = FALSE;
+
+  g_return_val_if_fail (GST_IS_DEVICE_PROVIDER (provider), FALSE);
+
+  g_mutex_lock (&provider->priv->start_lock);
+  started = provider->priv->started_count > 0;
+  g_mutex_unlock (&provider->priv->start_lock);
+  return started;
+}
